'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var Format = require('./Format');

var _require = require('./utils'),
    adjustLevel = _require.adjustLevel,
    LEVELS = _require.LEVELS,
    ERROR = _require.ERROR,
    WARN = _require.WARN,
    INFO = _require.INFO,
    DEBUG = _require.DEBUG;

var Namespaces = require('./Namespaces');

function LogBase(name, opts) {
  Object.assign(this, {
    name: name,
    opts: opts,
    _enabled: {},
    formatter: new Format(opts)
  });
  this.enable();
}

LogBase.prototype = {
  _serverinfo: function _serverinfo() {},


  /**
  * @return {object} json object
  */
  _formatJson: function _formatJson(level, _args) {
    var args = _args.slice(); // work on copy
    var opts = this.opts;

    var o = {
      level: level,
      name: this.name

      // remove object only formats
    };if (/^\s*%[oOj]\s*$/.test(args[0])) {
      args.shift();
    }
    // format a string based message
    if (['undefined', 'number', 'boolean'].indexOf(_typeof(args[0])) !== -1) {
      o.msg = args.shift();
    } else if (typeof args[0] === 'string') {
      var _formatter;

      args = (_formatter = this.formatter).format.apply(_formatter, _toConsumableArray(args));
      o.msg = args.shift();
    }
    // check for error
    if (args[0] instanceof Error) {
      var err = args.shift();
      o.err = {
        name: err.name,
        message: err.message,
        stack: err.stack
        // append other keys
      };Object.keys(err).forEach(function (key) {
        o[key] = err[key];
      });
    }
    if (_typeof(args[0]) === 'object') {
      o.body = args.shift();
    }
    // any other arguments are added to `.args`
    if (args.length) {
      o.args = args;
    }
    // we put serverinfo and date at the end of the object
    // for not viewing the same info e.g. if in tty
    if (opts.serverinfo) this._serverinfo(o);
    if (!opts.hideDate) o.time = new Date().toISOString();
    // diff info is not in humanized form
    o.diff = this.diff;

    return o;
  },
  _diff: function _diff() {
    var curr = Date.now();
    var prev = this.prev || curr;
    this.diff = curr - prev;
    this.prev = curr;
  },
  _log: function _log() /* level, args */{
    throw new Error('needs implementation');
  },
  enable: function enable(namespaces) {
    var _this = this;

    namespaces = namespaces || this.opts.namespaces;
    var namespace = new Namespaces(namespaces);
    this._enabled = {}; // reset
    if (!namespaces && this.opts.level || namespace.isEnabled(this.name)) {
      LEVELS[adjustLevel(this.opts.level, DEBUG)].forEach(function (level) {
        return _this._enabled[level] = true;
      });
    }
  },
  enabled: function enabled(level) {
    level = adjustLevel(level, ERROR);
    return !!this._enabled[level];
  },
  debug: function debug() {
    if (!this._enabled.DEBUG) return;

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return this._log(DEBUG, args);
  },
  info: function info() {
    if (!this._enabled.INFO) return;

    for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      args[_key2] = arguments[_key2];
    }

    return this._log(INFO, args);
  },
  warn: function warn() {
    if (!this._enabled.WARN) return;

    for (var _len3 = arguments.length, args = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
      args[_key3] = arguments[_key3];
    }

    return this._log(WARN, args);
  },
  error: function error() {
    if (!this._enabled.ERROR) return;

    for (var _len4 = arguments.length, args = Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
      args[_key4] = arguments[_key4];
    }

    return this._log(ERROR, args);
  }
};

module.exports = LogBase;